#ifndef  __XVEC_H__
#define  __XVEC_H__

//#include <algorithm>
#include <iostream>
//#include <stddef.h>
#include <math.h>


// column type 3 vectors

template<class real_t> 
class XVecT {
public:
  enum{ X = 0, Y = 1, Z = 2 };

  XVecT(void) {}
  XVecT(real_t f ) { m_v[X] = m_v[Y] = m_v[Z] = f; }
  XVecT(const XVecT& c) { 
    m_v[X] = c.m_v[X];
    m_v[Y] = c.m_v[Y];
    m_v[Z] = c.m_v[Z]; 
  }
  XVecT( const real_t& a, const real_t& b, const real_t& c ) { 
    m_v[X] = a;
    m_v[Y] = b;
    m_v[Z] = c; 
  }
  XVecT( const real_t* a ) { 
    m_v[X] = a[X];
    m_v[Y] = a[Y];
    m_v[Z] = a[Z]; 
  }

  operator real_t*( void ) { 
    return &m_v[0]; 
  }
  operator const real_t*( void ) const { 
    return &m_v[0]; 
  }

  bool operator==(const XVecT& c) const { 
    return m_v[X]==c.m_v[X] && m_v[Y]==c.m_v[Y] && m_v[Z]==c.m_v[Z];
  }

  XVecT& operator=( const XVecT& c ) { 
    m_v[X] = c.m_v[X];
    m_v[Y] = c.m_v[Y];
    m_v[Z] = c.m_v[Z]; 
    return *this; 
  }
  
  XVecT operator+( const XVecT& c ) const { 
    return XVecT( m_v[X] + c.m_v[X], m_v[Y] + c.m_v[Y], m_v[Z] + c.m_v[Z] ); 
  }
  XVecT operator-( const XVecT& c ) const { 
    return XVecT( m_v[X] - c.m_v[X], m_v[Y] - c.m_v[Y], m_v[Z] - c.m_v[Z] ); 
  }
  XVecT operator*( const real_t s ) const { 
    return XVecT( s * m_v[X], s * m_v[Y], s * m_v[Z] ); 
  }
  friend XVecT operator*( const real_t s, const XVecT& c ) { 
    return XVecT( s * c.m_v[X], s * c.m_v[Y], s * c.m_v[Z] ); 
  }
  XVecT operator/( const real_t s ) const { 
    return XVecT( m_v[X] / s, m_v[Y] / s, m_v[Z] / s ); 
  }

  XVecT& operator+=( const XVecT& c ) { 
    m_v[X] += c.m_v[X];
    m_v[Y] += c.m_v[Y];
    m_v[Z] += c.m_v[Z]; 
    return *this; 
  }
  XVecT& operator-=( const XVecT& c ) { 
    m_v[X] -= c.m_v[X];
    m_v[Y] -= c.m_v[Y];
    m_v[Z] -= c.m_v[Z]; 
    return *this; 
  }
  XVecT& operator*=( const real_t s ) { 
    m_v[X] *= s;
    m_v[Y] *= s;
    m_v[Z] *= s; 
    return *this; 
  }
  XVecT& operator/=( const real_t s ) { 
    m_v[X] /= s;
    m_v[Y] /= s;
    m_v[Z] /= s; 
    return *this; 
  }
  XVecT operator-( void ) const { 
    return XVecT( -m_v[X], -m_v[Y], -m_v[Z] ); 
  }

  //stupid but useful
  XVecT operator*(const XVecT& c)const{
    return XVecT(m_v[X]*c.m_v[X], m_v[Y]*c.m_v[Y], m_v[Z]*c.m_v[Z]);
  }
  XVecT operator/(const XVecT& c)const{
    return XVecT(m_v[X]/c.m_v[X], m_v[Y]/c.m_v[Y], m_v[Z]/c.m_v[Z]);
  }

  real_t& x( void ) { return m_v[X]; }
  real_t x( void ) const { return m_v[X]; }
  real_t& y( void ) { return m_v[Y]; }
  real_t y( void ) const { return m_v[Y]; }
  real_t& z( void ) { return m_v[Z]; }
  real_t z( void ) const { return m_v[Z]; }
  real_t& operator() (const int i)      { return m_v[i]; }
  real_t operator() (const int i) const { return m_v[i]; }

  const real_t& ref() const {
    return m_v[0];
  }

  XVecT Min( const XVecT& o ) const
  { 
#ifdef _MSC_VER
		real_t a = __min( m_v[X], o.m_v[X] );
    real_t b = __min( m_v[Y], o.m_v[Y] );
    real_t c = __min( m_v[Z], o.m_v[Z] );
#else
		real_t a = ::Min( m_v[X], o.m_v[X] );
    real_t b = ::Min( m_v[Y], o.m_v[Y] );
    real_t c = ::Min( m_v[Z], o.m_v[Z] );
#endif
    return XVecT( a, b, c ); 
	}
  
  XVecT Max( const XVecT& o ) const
  {    
#ifdef _MSC_VER
		real_t a = __max( m_v[X], o.m_v[X] );
    real_t b = __max( m_v[Y], o.m_v[Y] );
    real_t c = __max( m_v[Z], o.m_v[Z] );
#else
		real_t a = ::Max( m_v[X], o.m_v[X] );
    real_t b = ::Max( m_v[Y], o.m_v[Y] );
    real_t c = ::Max( m_v[Z], o.m_v[Z] );
#endif
    return XVecT( a, b, c ); 
	}

  void bbox( XVecT& cmin, XVecT& cmax ){
    if( m_v[X] < cmin.m_v[X] ) cmin.m_v[X] = m_v[X];
    if( m_v[X] > cmax.m_v[X] ) cmax.m_v[X] = m_v[X];
    if( m_v[Y] < cmin.m_v[Y] ) cmin.m_v[Y] = m_v[Y];
    if( m_v[Y] > cmax.m_v[Y] ) cmax.m_v[Y] = m_v[Y];
    if( m_v[Z] < cmin.m_v[Z] ) cmin.m_v[Z] = m_v[Z];
    if( m_v[Z] > cmax.m_v[Z] ) cmax.m_v[Z] = m_v[Z];
  }

  real_t dot( const XVecT& c ) const { 
    return m_v[X] * c.m_v[X] + m_v[Y] * c.m_v[Y] + m_v[Z] * c.m_v[Z]; 
  }
  real_t dot( void ) const { 
    return m_v[X] * m_v[X] + m_v[Y] * m_v[Y] + m_v[Z] * m_v[Z]; 
  }
  XVecT cross( const XVecT& c ) const { 
    return XVecT( m_v[Y] * c.m_v[Z] - m_v[Z] * c.m_v[Y],
                  m_v[Z] * c.m_v[X] - m_v[X] * c.m_v[Z],
                  m_v[X] * c.m_v[Y] - m_v[Y] * c.m_v[X] ); 
  }
  real_t l1( void ) const { 
    real_t a = fabs( m_v[X] ); 
    a += fabs( m_v[Y] ); 
    return a + fabs( m_v[Z] ); 
  }
  real_t l2( void ) const { 
    return sqrtf( dot() ); 
  }

  real_t dist( const XVecT& c ) const {
    return ( *this - c ).l2(); 
  }

protected:
  real_t  m_v[3];
};

template<class real_t> 
std::ostream& operator<<( std::ostream& os, const XVecT<real_t>& c )
{
  os << c.x() << " " << c.y() << " " << c.z();
  return os;
}

template<class real_t> 
std::istream&
operator>>( std::istream& is, XVecT<real_t>& f )
{
  return is >> f.x() >> f.y() >> f.z();
}

typedef XVecT<double> XVecd;
typedef XVecT<float> XVecf;

#endif  /* __CVECD_H__ */
